@@ -103,7 +103,7 @@ class DispatcherExecutor(Executor):
 
         if self.remote_command is None:
             self.remote_command = template.command
-        map_cmd = "sed -i \\\"s#/tmp#$(pwd)/tmp#g\\\" script && " if self.map_tmp_dir else ""
+        map_cmd = "if [ \\\"$(head -n 1 script)\\\" != \\\"# modified by dflow\\\" ]; then sed -i \\\"s#/tmp#$(pwd)/tmp#g\\\" script; sed -i \\\"1i # modified by dflow\\\" script; fi && " if self.map_tmp_dir else ""
         self.task_dict["command"] = "%s %s script" % (map_cmd, "".join(self.remote_command))
         self.task_dict["forward_files"] = ["script"]
         for art in template.inputs.artifacts.values():
@@ -117,15 +117,15 @@ class DispatcherExecutor(Executor):
         for par in template.outputs.parameters.values():
             if par.save_as_artifact:
                 self.task_dict["backward_files"].append("./" + par.path)
-            else:
+            elif par.value_from_path is not None:
                 self.task_dict["backward_files"].append("./" + par.value_from_path)
 
         new_template.script = "import os\n"
         new_template.script += "os.chdir('/')\n"
         new_template.script += "with open('script', 'w') as f:\n"
-        new_template.script += "    f.write(r'''\n"
+        new_template.script += "    f.write(r\"\"\"\n"
         new_template.script += template.script
-        new_template.script += "''')\n"
+        new_template.script += "\"\"\")\n"
 
         new_template.script += "import json\n"
         new_template.script += "from dpdispatcher import Machine, Resources, Task, Submission\n"
